open Mlpost
open Metapost
open Command
open Picture
open Path
open Num
open Num.Infix
open Helpers
open Box
open Color

let pen = Pen.transform [Transform.scaled (bp 3.)] Pen.default

let tabular l =
  "\\begin{tabular}{c}" ^
    String.concat "\\\\\n" l ^
    "\\end{tabular}"

let rec listn n x = if n = 0 then [] else x :: listn (n-1) x

let rose = rgb8 255 204 184
let wbox = round_rect ~stroke:(Some orange) ~fill:lightyellow ~dy:zero
let obox = round_rect ~stroke:None ~fill:rose ~dy:zero
let tbox = round_rect ~stroke:None ~fill:lightblue ~dy:zero

let overview =
  let twhyml = tex "\\bf ~~~~~~~~~~~~~~WhyML" in
  let twhy = tex "\\bf ~~~~~~~~~~~~~~~~~~Why" in
  let vcgen = obox (tex "VCgen") in
  let whyml =
    vbox ~pos:`Center ~padding:(bp 5.)
      [twhyml;
       vcgen;
       empty ()]
  in
  let trans = obox (tex "transform/translate") in
  let run = obox (tex "print/run") in
  let why =
    vbox ~pos:`Center ~padding:(bp 10.)
      [twhy;
       trans;
       run;
       empty ()
      ]
  in
  let mlw_file = tbox (tex "file.mlw") in
  let why_file = tbox (tex "file.why") in
  let coq = tbox (tex "Coq") in
  let alt_ergo = tbox (tex "Alt-Ergo") in
  let gappa = tbox (tex "Gappa") in
  let z3 = tbox (tex "Z3") in
  let etc = tex ~dx:zero "\\!\\!\\!etc." in
  let b =
    vbox ~padding:(bp 15.)
      [vbox ~padding:(bp 15.)
         [hbox ~padding:(bp 20.) ~pos:`Bot
            [why_file; vbox  ~padding:(bp 15.) [mlw_file; wbox whyml]
            ];
	  wbox why];
       hbox ~padding:(bp 10.) [coq; alt_ergo; gappa; z3; etc];
      ]
  in
  let head = Arrow.head_classic ~color:red ~pen in
  let kind =
    Arrow.add_line ~color:red  ~pen (Arrow.add_head ~head Arrow.empty)
  in
  let arrow ?ind ?outd x y =
    Arrow.box_to_box ~kind ?ind ?outd (sub x b) (sub y b) ~sep:(bp 5.)
  in
  draw b ++ arrow vcgen trans ++ arrow trans run ++
  arrow mlw_file vcgen ++ arrow why_file trans ++
  arrow run coq ++ arrow run alt_ergo ++ arrow run gappa ++ arrow run z3

let () = Metapost.emit "overview" overview

open Tree

let bin s = bin ~arrow_style:Undirected ~cs:(bp 20.) (tex s)
let empty = leaf (empty ())
let leaf s = bin s empty empty

let same_fringe =
  let t1 = bin "8" (bin "3" (leaf "1") (bin "5" (leaf "4") empty)) empty in
  let t2 = bin "4" (bin "1" empty (leaf "3")) (bin "8" (leaf "5") empty) in
  hbox ~pos:`Top ~padding:(bp 40.) [to_box t1; to_box t2]

let () = Metapost.emit "same_fringe" (Box.draw same_fringe)

let triangle ?(pos=`Lowright) ?(dx=(bp 10.)) ?(height=(bp 25.)) s h =
  let text = Box.tex s in
  let w = Box.width text +/ dx +/ dx in
  let path = Path.pathn ~style:jLine ~cycle:jLine
    [zero, zero; w, zero; w /./ 2., height] 
  in
  Box.pic ~stroke:None
    (Picture.make 
      (Command.draw path ++ 
       Command.label ~pos:`Top (Picture.tex s) (Point.pt (w /./ 2., zero)) ++
       Command.label ~pos:`Bot (Picture.tex h) (Point.pt (w /./ 2., zero))))

let line x y =
  let p = cpath_left x (Point.shift (Point.pt (zero, bp (-2.))) (north y)) in
  Command.draw p

let enum =
  let x1 = circle (tex "$x_1$") in
  let x2 = shift (Point.pt (bp 30., bp 30.)) (circle (tex "$x_2$")) in
  let xi = shift (Point.pt (bp 55., bp 55.)) (tex "...") in
  let xn = shift (Point.pt (bp 80., bp 80.)) (circle (tex "$x_n$")) in
  let t1 = shift (Point.pt (bp 10., bp (-45.))) (triangle "$t_1$" "") in
  let t2 = shift (Point.pt (bp 40., bp (-15.))) (triangle "$t_2$" "") in
  let tn = shift (Point.pt (bp 90., bp (35.))) (triangle "$t_n$" "") in
  Box.draw x1 ++ Box.draw x2 ++ Box.draw xi ++ Box.draw xn ++
  Helpers.box_arrow x1 x2 ~sep:(bp 5.) ++
  Helpers.box_arrow x2 xi ~sep:(bp 5.) ++
  Helpers.box_arrow xi xn ~sep:(bp 5.) ++
  Box.draw t1 ++ Box.draw t2 ++ Box.draw tn ++
  line x1 t1 ++
  line x2 t2 ++
  line xn tn ++
  nop

let () = Metapost.emit "enum" enum

let enum1 =
  let x1 = circle (tex "1") in
  let x3 = shift (Point.pt (bp 30., bp 30.)) (circle (tex "3")) in
  let x8 = shift (Point.pt (bp 60., bp 60.)) (circle (tex "8")) in
  let t1 = shift (Point.pt (bp 10., bp (-15.))) (to_box empty) in
  let t3 = shift (Point.pt (bp 50., bp (15.))) (to_box (bin "5" (leaf "4") empty)) in
  let t8 = shift (Point.pt (bp 70., bp (50.))) (to_box empty) in
  Box.draw x1 ++ Box.draw x3 ++ Box.draw x8 ++
  Helpers.box_arrow x1 x3 ~sep:(bp 5.) ++
  Helpers.box_arrow x3 x8 ~sep:(bp 5.) ++
  Box.draw t1 ++ Box.draw t3 ++ Box.draw t8 ++
  line x1 t1 ++ line x3 t3 ++ line x8 t8 ++
  nop

let () = Metapost.emit "enum1" enum1


let enum2 =
  let x1 = circle (tex "1") in
  let x4 = shift (Point.pt (bp 40., bp 40.)) (circle (tex "4")) in
  let t1 = shift (Point.pt (bp 20., bp (-15.))) (to_box (bin "3" empty empty)) in
  let t4 = shift (Point.pt (bp 60., bp (25.))) (to_box (bin "8" (leaf "5") empty)) in
  Box.draw x1 ++ Box.draw x4 ++
  Helpers.box_arrow x1 x4 ~sep:(bp 5.) ++
  Box.draw t1 ++ Box.draw t4 ++
  line x1 t1 ++ line x4 t4 ++ 
  nop

let () = Metapost.emit "enum2" enum2


(*
Local Variables:
compile-command: "mlpost -native -latex why3.tex -xpdf figures2.ml"
End:
*)

